#!/usr/bin/env python3
# -*- coding: iso-8859-13 -*-
#
# Autorius: Laimonas Vėbra, 2010
#
"""
sort.py -- surikiuoja (pagal lokalę) failo arba STDIN eilutes/žodžius.
Moka išvalyti komentarus, pašalinti besikartojančias eilutes, "gudriai" 
rikiuoti, t.y. atsižvelgti į tam tikrą failo struktūrą (kol kas moka 
rikiuoti užkomentuotuos žodžius, ignoruojant komentarą)

Usage: 
	./sort.py [options] file > sorted
	cat file | sort.py [options] > sorted

Options:
	see usage()
"""

import os, sys
import fileinput
import getopt
from locale import setlocale, getdefaultlocale, LC_COLLATE, strxfrm


# sets modulis paseno ir nuo v2.6+ sistemoje (built-in) jį keičia
# set/frozenset tipai; importuojant pasenusį -- įspėjama (warning).
if sys.version_info < (2, 6):
    from sets import Set


def _set(arg=''):
    if sys.version_info < (2, 6):
        return Set(arg)  
    else:
        return set(arg)



def usage():
	print \
"""
Usage: 
	sort.py [-h,--help] [-s,--strip] [-u,--unique] file|STDIN

Options:
	-h, --help      Display this help message;
        -c, --clean     Clean/strip all comments (#);
        -s, --smart     Smart sort (inc. commented words);
        -u, --unique    Remove duplicate lines.
"""



try:
	opts, rargs = getopt.getopt(sys.argv[1:], 
			"hcsu", ["help", "clean", "smart", "unique"])

except getopt.GetoptError:
	usage()
	sys.exit(2)               


unique_lines = 0
strip_comments = 0
smart_sort = 0


for opt, arg in opts:
	if opt in ("-h", "--help"):
            usage()                     
            sys.exit(2)


        if opt in ("-c", "--clean"):
            strip_comments = 1

        if opt in ("-s", "--smart"):
            smart_sort = 1

        if opt in ("-u", "--unique"):
            unique_lines = 1



# win lokalės atpažinimo/nustatymo problemos...
locale = getdefaultlocale()
if os.name is "nt":
	locale = "Lithuanian"

try:
	setlocale(LC_COLLATE, locale)
except:
	sys.stderr.write("Could not set locale\n")


def _tsmart(s):
	""" Smart (custom) transfrom; strxfrm() """
	# Ignoruojame komentaro simbolį ir rikiuojame pagal žodį už jo
	if s.startswith("#"): s = s[1:]
	return strxfrm(s)
		


def sort(lines):
	words = []
	uset = _set()	    	

	for line in lines:
	        line = line.strip()

		if (smart_sort or strip_comments):
		        lwords = line.split("#")
			word1 = lwords[0].strip()
			if len(lwords) >= 2:
				word2 = lwords[1]  
			else: 
				word2 = None
		

			if not word1:
				if (smart_sort and word2):
					# XXX prielaida:
					# po komentaro tarpas; tai komentarų 
					# bloko (ar šiaip) komentaras, bet ne 
					# užkomentuotas _žodyno_ žodis.
					if word2.startswith((" ", "\t")):
						if strip_comments: continue

				else: continue				

			else: 
				if strip_comments: line = word1


                       
		if unique_lines:
			if line not in uset: 
				words.append(line)
			uset.add(line)
		else:
			words.append(line)			
	
	
	if smart_sort:
		words.sort(key=_tsmart)
	else:
		words.sort(key=strxfrm)
	

	for line in words: 
		print line



if __name__ == "__main__":
    sort(fileinput.input(rargs))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (c) 2016, Laimonas Vėbra
# All rights reserved.
#
# This program is licensed under the Simplified BSD License.
# See <http://www.opensource.org/licenses/bsd-license>
#
"""Converts text from one encoding to another encoding.

Poor man's iconv. Does not buffer/block stdin on posix/linux, so
line-based two-way IPC (with a coprocess) is possible, unlike
the original iconv (fread), which blocks until EOF.
"""

import sys
import getopt
import locale

__version__ = '1.0a'
__program__ = 'iconv.py'

__usage_short = (
"""Usage:
  %s [OPTION...] [-f ENCODING] [-t ENCODING] [INPUTFILE...]
""") % __program__

__usage_full = (__usage_short +
"""
Converts text from one encoding to another encoding.

Options controlling the input and output format:
  -f ENCODING, --from-code=ENCODING
                              the encoding of the input
  -t ENCODING, --to-code=ENCODING
                              the encoding of the output
Options controlling conversion problems:
  -c                          discard unconvertible characters

Informative output:
  --help                      display this help and exit
  --version                   output version information and exit
""")


try:
    opts, files = getopt.getopt(sys.argv[1:],
                        "cf:t:",
                        ["help", "from-code", "to-code", "version"])
except getopt.GetoptError as e:
    print(e)
    print(__usage_short)
    sys.exit(2)

errors='strict'
src_enc = dst_enc = locale.getpreferredencoding()

for opt, arg in opts:
    if opt in ("--help"):
        print(__usage_full)
        sys.exit()
    elif opt in ("--version"):
        print(__program__, __version__)
        sys.exit()
    elif opt in ("-c"):
        errors='ignore'
    elif opt in ("-f", "--from-code"):
        src_enc = arg
    elif opt in ("-t", "--to-code"):
        dst_enc = arg
    else:
        assert False, "unhandled option"

if sys.version_info >= (3,1):
    sys.stdin = sys.stdin.detach()
    sys.stdout = sys.stdout.detach()

def convert(line):
    return line.decode(src_enc, errors).encode(dst_enc, errors)

if files:
    for f in files:
        f = open(f, mode='rb')
        for line in f:
            sys.stdout.write(convert(line))
        f.close()
else:
    while True:
        line = sys.stdin.readline()
        if not line:
            break
        sys.stdout.write(convert(line))



#!/usr/bin/env python3
"""
Priemonės žodžių ieškoti VDU KLC tekstyne per webinį interfeisą.

$Id: donelaitis.py,v 1.1 2003/06/11 10:06:48 alga Exp $
"""
import urllib
import re
import ispell
import sys

def parseTable(html):
    tag = re.compile('<.*?>')
    td  = re.compile('<td .*?>')
    tr  = re.compile('<tr .*?>')

    html = td.sub('<td>', html)
    html = tr.sub('<tr>', html)

    table = html[html.find('<table'):html.find('</table')]
    row_index = table.find('<tr')
    rows = table.split('<tr>')[1:]
    table = ()
    for row in rows:
        cols = row.split('<td>')[1:]
        result = ()
        for col in  cols:
            col = tag.sub('', col)
            col = col.strip()
            result += (col,)
        table += (result,)
    return table

class CorpusWord:

    def __init__(self, word, prefetch=True):
        self.word = word
        self.url = \
                 "http://donelaitis.vdu.lt/cgi-bin/find2.cgi?a=%s&kontek=&l=2"
        self.html = None
        if prefetch:
            self.fetch()

    def fetch(self):
        """Atsisiunčia ir išlukštena tekstyno duomenis"""
        if self.html is None:
            self.html = urllib.urlopen(self.url % self.word).read()
        self.data = parseTable(self.html)

    def totalMatches(self):
        """Grąžina kiek viso žodžio pasirodymų yra tekstyne"""
        result = 0
        for row in self.data:
            try:
                result += int(row[1])
            except ValueError:
                pass
        return result


def sortWords(dict=None, all_forms=False):
    """Visus žodžius iš žodyno ieškom tekstyne ir spausdinam su
    pasitaikymų dažniais."""

    # XXX: Unit tests!

    if dict is None:
        dict = open(sys.argv[1])

    results = []
    for line in dict.readlines():

        line = line.strip()
        word = ispell.splitEntry(line)[0]

        if all_forms:
            words = ispell.expand(line)
        else:
            words = [word]

        for word in words:
            if word:
                print word,
                w = CorpusWord(word)
                c = w.totalMatches()
                results.append((c, word))

    results.sort()
    return ["%d\t%s" % (c, w) for c, w in results]

if __name__ == '__main__':
    print "\n".join(sortWords())



#!/usr/bin/env python
"""
Testai VDU KLC žodyno interfeisui.

$Id: test_donelaitis.py,v 1.1 2003/06/11 10:06:48 alga Exp $
"""

import unittest

class TestCorpusWord(unittest.TestCase):

    def test_parseTable(self):
        from donelaitis import parseTable

        table = """
        <html a=b><body>
        <table width=100%>
          <tr color="red">
            <td foo>aaa </td>
            <td> bbb  </td>
            <td foo> ccc </td>
          </tr>
          <tr>
            <td foo> ddd </td>
            <td> eee </td>
            <td foo> fff </td>
          </tr>
        </table>
        </body></html>
        """

        result = (('aaa', 'bbb', 'ccc'), ('ddd', 'eee', 'fff'))

        self.assertEquals(parseTable(table), result)
        self.assertEquals(parseTable("foo"), ())

    def test_fetch(self):
        from donelaitis import CorpusWord

        c = CorpusWord("mama", prefetch=False)
        c.html = """
        <html a=b><body>
        <table width=100%>
          <tr color="red">
            <td foo>aaa </td>
            <td> bbb  </td>
            <td foo> ccc </td>
          </tr>
          <tr>
            <td foo> ddd </td>
            <td> eee </td>
            <td foo> fff </td>
          </tr>
        </table>
        </body></html>
        """
        result = (('aaa', 'bbb', 'ccc'), ('ddd', 'eee', 'fff'))

        c.fetch()
        self.assertEqual(c.data, result)

    def test_totalMatches(self):
        from donelaitis import CorpusWord
        c = CorpusWord("mama", prefetch=False)
        c.data = (('grožinėj', '12', '1234', '12345'),
                  ('publicistinėj', '13', '1234', '12345'),
                  ('viso', '', '1234', '12345'))
        self.assertEqual(c.totalMatches(), 25)


class TestIspell(unittest.TestCase):

    def test_entry(self):
        from ispell import splitEntry

        word, flags = splitEntry("namas/D\n")
        self.assertEqual(word, "namas")
        self.assertEqual(flags, "D")

        word, flags = splitEntry("viščiukas/D\n")
        self.assertEqual(word, "viščiukas")
        self.assertEqual(flags, "D")

        word, flags = splitEntry("geras/AQN\n")
        self.assertEqual(word, "geras")
        self.assertEqual(flags, "AQN")

        word, flags = splitEntry(" # Čia komentaras\n")
        self.assertEqual(word, "")
        self.assertEqual(flags, "")

        word, flags = splitEntry("kažkada")
        self.assertEqual(word, "kažkada")
        self.assertEqual(flags, "")

        word, flags = splitEntry("kažkada # nekaitomas")
        self.assertEqual(word, "kažkada")
        self.assertEqual(flags, "")

    def test_expand(self):
        from ispell import expand

        entry = "namas/D"

        result = ['namai', 'namais', 'namams', 'namas', 'name',
                  'namo', 'namu', 'namui', 'namuose', 'namus', 'namą',
                  'namų']

        self.assertEqual(expand(entry), result)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
"""
Priemonės kreiptis į ispell žodyną.

$Id: ispell.py,v 1.1 2003/06/11 10:06:48 alga Exp $
"""
import os

def splitEntry(line):
    line = line.strip()
    index = line.find("#")
    if index >= 0:
        line = line[:index]
        line = line.strip()
    index = line.find("/")
    if index >= 0:
        word = line[:index]
        flags = line[index+1:]
        return word, flags
    return line, ""

def expand(line):
    """Grąžina surūšiuotą sąrašą žodžių, į kuriuos išsiskleidžia eilutė"""

    input, output = os.popen2("ispell -e")
    #import pdb; pdb.set_trace()
    input.write(line)
    input.flush()
    input.close()
    words = output.read()
    output.close()
    words = words.split()
    words.sort()
    return words



#!/usr/bin/env python3
# -*- coding: iso-8859-13 -*-
#
# Autorius: Albertas Agejevas, 2003
# Koregavo: Laimonas Vėbra, 2010-2016
#
# Veikia su Python v2.3+, v3.0+
#
"""
ispell-lt projekto/žodyno įrankis.
Suglaudžia/suskliaudžia pasikartojančius žodžius (suliejant jų afiksų
žymas, jei tokių turi), o taip pat priešdėlinius veiksmažodžius, pvz.:
    pa|eina, nu|eina, at|eina, ... -> eina/bef...

ir išveda surikiuotų žodžių sąrašą, tinkamą galutiniam žodynui.

Žodžiai skliaudžiami tik suderinamose (kalbos dalių) grupėse (dabar
tai: veiksmažodžiai, būdvardžiai ir likę).

Naudojimas:
    ./sutrauka žodynas.txt > sutraukta.txt
    cat žodynas.txt | ./sutrauka > sutraukta.txt

"""
import os
import sys
import locale
import fileinput
from itertools import chain


enc = "ISO8859-13"
loc = "lt_LT" + "." + enc

# Windows setlocale() nepriima POSIX lokalės
if os.name is "nt":
    loc = "Lithuanian"

_setlocale_failed = False
try:
    locale.setlocale(locale.LC_COLLATE, loc)
except:
    _setlocale_failed = True
    sys.stderr.write(
        "Could not set locale '%s', default: '%s'. "
        "Won't be able to sort dictionary words correctly.\n"
        % (loc, locale.getdefaultlocale()))

# Nuo v2.4 set tipai built-in, o sets modulis deprecated nuo v2.6
if sys.version_info < (2, 4):
    from sets import Set
    set = Set

# Py2 ir Py3 dict iteratorių suderinimas
if sys.version_info < (3,):
    items = dict.iteritems
else:
    items = dict.items


# global stat vars: constringed words and saved bytes count
c_wcount = 0
c_bsaved = 0

prefixes = (
    ("a", "ap"), ("a", "api"),
    ("b", "at"), ("b", "ati"),
    ("c", "į"),
    ("d", "iš"),
    ("e", "nu"),
    ("f", "pa"),
    ("g", "par"),
    ("h", "per"),
    ("i", "pra"),
    ("j", "pri"),
    ("k", "su"),
    ("l", "už"),
    ("m", "apsi"),
    ("n", "atsi"),
    ("o", "įsi"),
    ("p", "išsi"),
    ("q", "nusi"),
    ("r", "pasi"),
    ("s", "parsi"),
    ("t", "persi"),
    ("u", "prasi"),
    ("v", "prisi"),
    ("w", "susi"),
    ("x", "užsi"),
    )



def _stats(word, wflags, swflags, pverb=False):
    global c_wcount, c_bsaved
    # Statistika (sutaupyta žodžių ir vietos).
    #
    # Kiek sutaupoma vietos (bcount) suskliaudžiant žodį:
    # žodžio ilgis + bendrų žymų kiekis + _papildomai_ 1 arba 2 baitai,
    # priklausomai nuo varianto:
    #   - kai žodis be žymų arba priešd. veiksmažodis (pverb): '\n' (1)
    #   - visais kitais atvejais sutaupoma: '/', '\n' (2)
    c_wcount += 1

    if (pverb or not wflags):
        le = 2
    else:
        le = 1

    c_bsaved += len(word) + len(wflags & swflags) + le

def _msg(s, *args):
    if args:
        s = s % args
    sys.stderr.write(s)
    sys.stderr.flush()

def _progress(i, step=5000):
    if (i % step == 0):
        _msg('.')

def _sort(wlist):
    if _setlocale_failed:
        wlist.sort()
    elif sys.version_info < (2, 4):
        wlist.sort(locale.strcoll)
    elif sys.version_info < (3,):
        wlist.sort(cmp=locale.strcoll)
    elif sys.version_info >= (3,):
        from functools import cmp_to_key
        wlist.sort(key=cmp_to_key(locale.strcoll))


def sutrauka(lines, outfile=sys.stdout, myspell=True):
    i = 0
    adjes = {}
    verbs = {}
    words = {}
    wcount = 0

    # Skliaudžiamųjų žodžių klasės (pagal afiksų žymų rinkinius):
    vflags = set("TYEPRO")  # verb flags
    aflags = set("AB")      # adjective flags

    _msg("\n--- %s %s\nReading ", sys.argv[0], '-' * (55 - len(sys.argv[0])))

    for line in lines:
        _progress(lines.lineno())

        # Ignoruojamos tuščios ir komentaro eilutės.
        line = line.split("#")[0]
        line = line.strip()
        if not line:
            continue

        wcount += 1

        # Eilutė skeliama į žodį ir jo žymų rinkinį.
        sp = line.split("/")
        word = sp[0]
        if len(sp) > 1:
            wflags = set(sp[1])
        else:
            wflags = set()

        # Veiksmažodžiai ir būdvardžiai į atskirus dict.
        if vflags & wflags:
            d = verbs
        elif aflags & wflags:
            d = adjes
        else:
            d = words

        # Žodis pridedamas į dict arba jei jau yra -- suliejamos žymos
        swflags = d.get(word) # stored word flags
        if swflags is not None:
            _stats(word, wflags, swflags)
            swflags.update(wflags)
        else:
            d[word] = wflags


    _msg("\nProcessing ")

    # Priešdėlinių veiksmažodžių suskliaudimas.
    # XXX: dėl skirtingo py2 ir py3 dict vidinio eiliškumo, skiriasi ir
    # suglaudinimo rezultatas.
    # Neišspręsta problema: priklausomai nuo to, kurie žodžiai ir kokiu
    # eiliškumu išrenkami, skliaudžiant sudurtinių priešd. veiksmažodžius,
    # iš dict pašalinamas skliaudžiamasis žodis ir tai vėliau nebeleidžia
    # suskliausti kitų žodžių.
    # Pvz.:
    #    su|panašinti -> pa|našinti/k -> našinti/fk
    # vs
    #    pa|našinti -> našinti/f;
    #    (vėliau 'supanašinti' nebesuskliaudžiamas, nes nebėra 'panašinti')
    #
    # Norint vieningo rezultato su py2/py3, reikia surikiuoti sąrašą:
    #    lverbs = list(verbs); lverbs.sort()
    # nors problema išlieka: algoritmas ne visai korektiškai suskliaudžia
    # sudurtinių priešdėlių veiksmažodžius.
    for word in list(verbs):
        i += 1
        _progress(i)

        # Žodžio afiksų žymų rinkinys.
        wflags = verbs[word]

        # Kiekvienam žodyno žodžiui derinami/tikrinami visi priešdėliai.
        for pflag, pref in prefixes:

            if word.startswith(pref):
                # Jei pref sangrąžinis priešdėlis, tai žodis atmetus paprastąjį
                # (nesangrąžinį) priešdėlį, pvz.: iš{si}|urbia -> siurbia.
                # Kai toks žodis yra žodyne, tai situacija netriviali, nes
                # žodyne yra trys žodžio formos: su priešdėliu, be priešdėlio
                # ir be sangrąžinio priešdėlio.  Tampa nebeaišku kokį priešdėlį
                # (sangrąžinį ar ne) ir kokiam žodžiui pritaikyti; tokių
                # žodžių savaime suskliausti neįmanoma, pvz.:
                #     iš{si}|urbia, siurbia, urbia (iš|siurbia ar išsi|urbia?)
                #     at{si}|joja, sijoja, joja;   (at|sijoja ar atsi|joja?)
                #
                # Kol kas tokie žodžiai neskliaudžiami.
                if pref.endswith("si"):
                    # word without reflexive prefix part
                    wrp_word = word[len(pref)-2:]
                else:
                    wrp_word = None


                # Žodis be priešdėlio, pvz.: per|šoko -> šoko.
                # (word without prefix)
                wp_word = word[len(pref):]
                wp_wflags = verbs.get(wp_word)

                if (wp_wflags is not None and wrp_word not in verbs):
                    # Skliaudžiant priešdėlinius veiksmažodžius su /N /S /X
                    # afiksų žymomis, dėl ispell apribojimo jungiant afiksus,
                    # prarandamos kelios priešdėlinės formos, pvz:
                    #
                    #   pavartyti/X  >  te|pa|vartyti, tebe|pa|vartyti,
                    #                   be|pa|vartyti, ...
                    # vs
                    #    vartyti/Xf  >  tevartyti, tebevartyti, bevartyti, ...
                    #
                    # Susitaikius su vykstančiu priešdėlinių formų:
                    #  [/N /S /X afiksai] {priešdėlis} žodis
                    # praradimu, žodynas suglaudinamas virš 50 kB.
                    #
                    # ARBA atvirkščiai: siekiant, kad nebūtų praradimų, kaip
                    # tik nereikėtų tokių žodžių (jei priešdėlinis žodis turi
                    # /S /X /N žymas) glaudinti.
                    _stats(word, wflags, wp_wflags, pverb=True)

                    # Suliejamos afiksų žymos ir pridedama priešdėlio žyma.
                    wp_wflags.update(wflags)
                    wp_wflags.add(pflag)

                    # Žodis sukliaustas (prie šakninio žodžio sulietos
                    # žymos, pridėta priešdėlio afikso žyma).  Pašaliname
                    # priešdėlinį žodį iš 'verbs' dict ir baigiame
                    # priešdėlių ciklą, nes priešdėliai unikalūs ir žodžio
                    # pradžia nebegali sutapti su jokiu kitu priešdėliu.
                    del verbs[word]
                    break

    # beafiksinių žodžių pašalinimas, jei jie yra kitose afiksinių klasėse
    for word, flags in items(words.copy()):
        if (not flags and (word in verbs or word in adjes)):
            _stats(word, flags, set())
            # _msg("Deleting %s\n", word)
            del words[word]
    
    wlist = []
    NS = set('NS')
    for word, flags in chain(items(words), items(verbs), items(adjes)):
        if flags:
            # /S perdengia /N, todėl abiejų nereikia
            if NS < flags:
                flags.remove('N')
            fl = list(flags)
            fl.sort()
            word += "/" + "".join(fl)

        wlist.append(word + '\n')

    _sort(wlist)

    _msg(" done.\nWords before: %d, words after: %d.\n"
            "(words constringed: %d, bytes saved: %d)\n%s\n",
             wcount, len(wlist), c_wcount, c_bsaved, '-' * 60)

    # myspell'o žodyno pradžioje -- žodžių kiekis.
    if myspell:
        outfile.write(len(wlist) + '\n')

    outfile.writelines(wlist)



if __name__ == "__main__":
    outfile = sys.stdout
    # Nuo v2.5+ fileinput galima nurodyti openhook'ą (dekodavimas iš
    # norimos koduotės). Aktualu tik py3 (py2 dirba su byte strings;
    # perkodavimas į unikodą nebūtinas), tačiau openhook'as neveikia
    # su stdin.
    if sys.version_info >= (3,):
        import io
        if not sys.argv[1:]:
            # jei nėra argumentų, tai duomenys iš stdin
            sys.stdin = io.TextIOWrapper(sys.stdin.buffer, encoding=enc)
        outfile = io.TextIOWrapper(sys.stdout.buffer, encoding=enc)
        _fileinput = fileinput.input(openhook=fileinput.hook_encoded(enc))
    else:
        _fileinput = fileinput.input()

    sutrauka(_fileinput, outfile=outfile, myspell=False)



#!/usr/bin/env python3
# -*- coding: iso-8859-13 -*-
'''
$Id: spell.py,v 1.4 2003/11/24 23:51:16 alga Exp $

Paleidimas:

  python3 spell.py [-m] [infile]

     -m     -- nurodo, kad žodžius reikia dėti į lietuviu.zodziai ir
               lietuviu.veiksmazodziai, o ne lietuviu.privatus

     infile -- failas, iš kurio skaityti po vieną žodžius (užuot
               skaičius iš klaviatūros)
'''

import sys
import os
from popen2 import popen2

def entry(prompt, default=None):
    """Input"""
    if default:
        prompt = "%s [%s]> " % (prompt, default)
    result = raw_input(prompt)
    result.strip()
    if not result:
        return default
    else:
        return result

def binary(prompt, default=False):
    """Klausimas, į kurį atsakymas yra taip arba ne"""
    default_str = default and "taip" or "ne"
    result = raw_input("%s [%s]> " % (prompt, default_str))
    result.strip()
    if not result:
        return default
    else:
        return result[0] in "TtYy"

def multi(prompt, *args):
    """Multiple choice.
    """
    prompt = "%s (%s)> " % (prompt,
                            ", ".join(["[%s]%s" % (choice[0], choice[1:])
                                       for choice in args]))
    result = raw_input(prompt)
    result.strip()
    if result:
        return result[0]

def minkst(zod):
    """Jei zod baigiasi d arba t, pakeičia dž arba č

    žod-is --> žodž-io
    kirt-is --> kirč-io
    sald-us --> saldž-iausias
    """
    if zod[-1] == 't':
        return zod[:-1] + "č"
    elif zod[-1] == 'd':
        return zod[:-1] + "dž"
    else:
        return zod

def daiktavardis(zodis):
    if zodis.endswith("is"):
        if binary("%s, %sio? " % (zodis, zodis[:-2])):
            return "%s/D" % zodis
        elif binary("Ar tai vyriškos giminės daiktavardis?"):
            return  "%s/V" % zodis
        elif binary("%s, %sų" % (zodis, zodis[:-2])):
            return  "%s/MK" % zodis
        else:
            print "Vadinasi, %s, %sių" % (zodis, minkst(zodis[:-2]))
            return  "%s/MI" % zodis
    elif zodis.endswith("uo"):
        if binary("Ar tai vyriškos giminės daiktavardis?"):
            return  "%s/V" % zodis
        else:
            return  "%s/M" % zodis
    else:
        return "%s/D" % zodis


priesdeliai = {
    "a": "ap",
    "b": "at",
    "c": "į",
    "d": "iš",
    "e": "nu",
    "f": "pa",
    "g": "par",
    "h": "per",
    "i": "pra",
    "j": "pri",
    "k": "su",
    "l": "už",
    "m": "apsi",
    "n": "atsi",
    "o": "įsi",
    "p": "išsi",
    "q": "nusi",
    "r": "pasi",
    "s": "parsi",
    "t": "persi",
    "u": "prasi",
    "v": "prisi",
    "w": "susi",
    "x": "užsi",
    }

def veiksmazodis(inf, es=None, but=None):
    print "Bendratis: %s" % inf
    if es is None:
        es = entry("Esamasis laikas (ką daro?)> ")
    if but is None:
        but = entry("Būtasis kartinis (ką darė?)> ")

    inf_flag = "T"
    es_flag = "E"

    # Gyti, gija, gijo, gis
    if inf[-3] == "y" and es[-3] == "y" and but[-3] == "i":
        inf_flag = "U"

    # Pūti, pūva, puvo, pus
    if inf[-3] == "ū" and es[-3] == "ū" and but[-3] == "u":
        inf_flag = "U"

    if inf.endswith("yti"):
        but_flag = "Y"
    else:
        but_flag = "P"

    if binary("%ss, %ssi, %ssi?" % (inf, es, but)):
        sangraza = "SX"
    else:
        sangraza = "NX"

    flagai = []
    for flag, priesdelis in priesdeliai.items():
        if inf[0] in "pb":
            fmt = "%si%s?"
        else:
            fmt = "%s%s?"
        if binary(fmt % (priesdelis, inf)):
            flagai.append(flag)
    flagai = "".join(flagai)

    if sangraza == "SX":
        return "\n".join(("%s/%s%s%s" % (inf, inf_flag, sangraza, flagai),
                          "%s/%s%s%s" % (es, es_flag, sangraza, flagai),
                          "%s/%s%s%s" % (but, but_flag, sangraza, flagai),
                          "%ss/%s" % (inf, inf_flag),
                          "%ssi/%s" % (es, es_flag),
                          "%ssi/%s" % (but, but_flag),
                          ))
    else:
        return "\n".join(("%s/%s%s%s" % (inf, inf_flag, sangraza, flagai),
                          "%s/%s%s%s" % (es, es_flag, sangraza, flagai),
                          "%s/%s%s%s" % (but, but_flag, sangraza, flagai),
                          ))

def kokybinis(zodis):
    """Paklausia, ar būdvardis kokybinis, ir grąžina "Q" arba "". """

    if binary("Ar jis yra kokybinis (%s, %sesnis, %siausias, )?" %
              (zodis, zodis[:-2], minkst(zodis[:-2]))):
        return "Q"
    else:
        return ""

def budvardis(zodis):
    if not zodis.endswith("is") or zodis in ('didis', 'didelis'):
        return "%s/ANQ" % zodis
    else:
        return "%s/BN" % zodis

def prideti(failas, zodis):
    """Prideda žodį į failą"""
    f = open(failas, "a")
    f.write(zodis)
    f.write("\n")
    f.close()

def patikrinti(zodis):
    child_out, child_in = popen2("ispell -d ./lietuviu -l")
    child_in.write(zodis)
    child_in.close()
    rez = child_out.read()
    return (len(rez) == 0)

def ispell_yra():
    out = os.popen("ispell -v")
    return out.close() is None

def gauti_zodi(file=None):
    if file is None:
        return entry("Įveskite pradinę žodžio formą> ")
    while 1:
        line = file.readline()
        line = line.strip()
        zodis = line.split("/")[0]
        if not zodis:
            continue
        if patikrinti(zodis):
            print "*** %s jau yra žodyne" % zodis
        else:
            break
    print ">>> %s " % line
    pataisymas = entry("[%s] >" % zodis)
    if pataisymas:
        zodis = pataisymas
    return zodis


def find_ispell_home():
    envpath = os.getenv("ISPELLLT")
    if envpath:
        os.chdir(envpath)
        return
    elif os.path.exists("lietuviu.dict"):
        return
    elif os.path.exists(os.path.join("..", "ispell-lt", "lietuviu.dict")):
        os.chdir(os.path.join("..", "ispell-lt"))
        return
    else:
        print "Nerandu ispell direktorijos -- " \
              "nustatykite ISPELLLT aplinkos kintamąjį į direktoriją,\n" \
              "kurioje yra lietuviu.dict ir kiti ispell failai."
        sys.exit(1)


def main(*args):
    infile = None
    maintainer_mode = False
    privatus = None
    turim_ispell = ispell_yra()

    find_ispell_home()

    for arg in args[1:]:
        if arg == '-m':
            maintainer_mode = True
        else:
            infile = open(arg)
            break
    try:
        if os.getenv("HOME"):
            privatus = os.path.join(os.getenv("HOME"), '.ispell_lietuviu')
        while True:
            zodis = gauti_zodi(infile)
            if turim_ispell and patikrinti(zodis):
                print "Beje, '%s' jau yra žodyne." % zodis
            dalis = multi("Kokia tai kalbos dalis?\n",
                          "veiksmažodis", "daiktavardis",
                          "budvardis", "nekaitoma")
            if maintainer_mode:
                dalys = { "d": (daiktavardis, "lietuviu.zodziai"),
                          "v": (veiksmazodis,  "lietuviu.veiksmazodziai"),
                          "b": (budvardis, "lietuviu.zodziai"),
                          "n": (lambda x: x, "lietuviu.zodziai"),
                          }
            else:
                dalys = { "d": (daiktavardis, "lietuviu.privatus"),
                          "v": (veiksmazodis,  "lietuviu.privatus"),
                          "b": (budvardis, "lietuviu.privatus"),
                          "n": (lambda x: x, "lietuviu.privatus"),
                          }
            try:
                dorokle, failas = dalys[dalis]
                irasas = dorokle(zodis)
                print "Dedam į %s:" % failas
                print irasas
                prideti(failas, irasas)
                if privatus:
                    prideti(privatus, irasas)
                print "Pridėta."
            except KeyError:
                print "Kas kas?"
    except (EOFError, KeyboardInterrupt):
        print
        return

if __name__ == "__main__":
    main(*sys.argv)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (c) 2002-2020 by Albertas Agejevas, Laimonas Vėbra
# All rights reserved.
#
# This program is licensed under the Simplified BSD License.
# See <http://www.opensource.org/licenses/bsd-license>
"""
Converts ispell affix file to OpenOffice's MySpell format.
See <https://www.openoffice.org/lingucomponent/affix.readme>
"""

import re
import sys
import getopt
import codecs

__version__ = '1.0a'
__program__ = 'ispell2myspell.py'

__usage_short = (
"""Usage:
  %s [OPTION...] ISPELL_AFF_FILE > MYSPELL_AFF_FILE
""") % __program__

__usage_full = (__usage_short +
"""
Converts ispell affix file to OpenOffice's MySpell format.

Options:
  -h, --help                  Display this help and exit
  -c MYCONFIG, --cfg=MYCONFIG
                              MySpell aff file without affix table.
                              (this file is included in MYSPELL_AFF_FILE)
  -d ENCODING, --dec=ENCODING
                              the encoding of the input (ISPELL_AFF_FILE).
                              (decode input from ENCODING to Unicode)
  -e ENCODING, --enc=ENCODING
                              the encoding of the output (MYSPELL_AFF_FILE).
                              (encode output from Unicode to ENCODING)
  -s, --sort
                              Sort affix flags. Default: original order.

  -v, --version               Output version information and exit.

Notes:
  MYCONFIG must be in utf-8 (it's converted to --enc).

  Input encoding can be specified in ISPELL_AFF_FILE comments according
  to PEP263 spec., e.g.:  # -*- coding: <encoding> -*-

  --dec overrides ISPELL_AFF_FILE encoding.

""")


class AffixTable:

    def __init__(self, ispell_aff_file, src_enc):
        self.affs = {}

        self.type_re = re.compile(r"(prefixes|suffixes)", re.I)
        self.flag_re = re.compile(r"flag([*~]?)(.):", re.I)
        self.rule_re = re.compile(r"(.+?)>(?:-(.+?),)?(.+)", re.I)

        self._in = codecs.open(ispell_aff_file, "r", src_enc)
        self._read_in()

    def _get_line(self):
        """
        Gets and returns next non-empty line (with comments stripped
        and whitespaces removed) or "" if EOF.
        """
        line = None
        while not line:
            line = self._in.readline()
            self.raw_line = line
            if line == "": # EOF
                break
            line = self._format(line)
        self.line = line
        return self.line

    def _format(self, s):
        "Removes spaces, comments (#.*) from a string."
        return re.sub(r"(#.*)|(\s+)", '', s)


    def _read_in(self):
        "Reads in and parses affix file"
        rules = []
        atype = None

        def _nul(s):
            if s:
                return s.strip("\"'") or 0
            return 0

        while self._get_line():

            m = self.type_re.search(self.line)
            if m:
                atype = m.group(1)
                if self.affs.get(atype) is None:
                    self.affs[atype] = {}
                    self.affs[atype]['flags'] = []
                continue
            if not atype:
                continue

            m = self.flag_re.search(self.line)
            if m:
                rules = []
                opt, flag = m.groups()
                # original affix flags order
                self.affs[atype]['flags'].append(flag)
                self.affs[atype][flag] = {
                    'rules': rules,
                    'combine': opt == '*',
                    'compound': opt == '~',
                    }
                continue

            if '>' in self.line:
                cond, repl = self.raw_line.lower().split('>', 1)
                # wrap non-separated chars in [] (alternatives in ispell)
                cond = re.sub(r"\s*([^\[\]\s]{2,})\s", r"[\1]", cond)
                m = self.rule_re.search(self._format(cond + '>' + repl))
                if m:
                    (cond, strip, add) = m.groups()
                    # null/empty string for myspell aff strip/add parts is 0
                    rule = (_nul(strip), _nul(add), cond)
                    if rule not in rules:
                        rules.append(rule)

    def dump_myspell_aff(self, myspell_cfg_file=None, dst_enc=None, sort=False):
        "Dumps MySpell affix table to stdout."

        out = sys.stdout
        if sys.version_info >= (3,0):
            out = sys.stdout.buffer
        out = codecs.getwriter(dst_enc)(out)

        ml = []
        if myspell_cfg_file:
            my_aff = codecs.open(myspell_cfg_file, "r", "utf-8")
            for line in my_aff:
                if re.match(r'^\s*#|SET', line, re.I):
                    continue
                ml.append(line)
            my_aff.close()

        out.write("SET %s\n" % dst_enc.upper())

        # dump base myspell config (aff) file
        out.writelines(ml)

        # dump myspell affix table
        for (_atype, atype) in [('prefixes', 'PFX'), ('suffixes', 'SFX')]:

            flags = self.affs[_atype]['flags']
            if sort:
                flags.sort()

            for flag in flags:
                aff = self.affs[_atype][flag]
                # affix header
                out.write("\n%s %s %s %d\n" %
                              (atype, flag, ("N", "Y")[aff['combine']],
                                   len(aff['rules'])))
                # affix rules
                for (strip, add, cond) in aff['rules']:
                    out.write("%s %s %-7s %-15s %s\n" %
                                  (atype, flag, strip, add, cond))

def _encoding_name(enc):
    codecs.lookup(enc)
    try:
        enc = codecs.lookup(enc).name
    except:
        pass
    return enc

def main():
    try:
        opts, files = getopt.getopt(sys.argv[1:],
                        "hc:d:e:sv",
                        ["help", "cfg=", "dec=", "enc=", "sort", "version"])
    except getopt.GetoptError:
        print(sys.exc_info()[1])
        print(__usage_short)
        sys.exit(2)

    sort_aff = False
    src_enc = dst_enc = None
    myspell_cfg_file =  None

    for opt, arg in opts:
        if opt in ("-h, --help"):
            print(__usage_full)
            sys.exit()
        elif opt in ("-v, --version"):
            print(__program__, __version__)
            sys.exit()
        elif opt in ("-c, --cfg"):
            myspell_cfg_file = arg
        elif opt in ("-d", "--dec"):
            src_enc = arg
        elif opt in ("-e", "--enc"):
            dst_enc = arg
        elif opt in ("-s", "--sort"):
            sort_aff = True
        else:
            assert False, "unhandled option"


    if len(files) >= 1:
        ispell_aff_file = files[0]
    else:
        print("Missing input argument (ISPELL_AFF_FILE).")
        print(__usage_short)
        sys.exit(1)

    # coding lookup (PEP263)
    if not src_enc:
        f = codecs.open(ispell_aff_file, "r", 'ascii', 'ignore')
        for n in range(2):
            m = re.match(r"^\s*#.*?coding[:=]\s*([-_.a-zA-Z0-9]+)",
                             f.readline(), re.I)
            if m:
                src_enc = m.group(1)
                break
        f.close()

    if src_enc:
        src_enc = _encoding_name(src_enc)
    else:
        print("Unknown ispell aff file encoding: "
                  "missing --enc arg and not specified in file.")
        sys.exit(3)

    if dst_enc:
        dst_enc = _encoding_name(dst_enc)
    else:
        dst_enc = src_enc
        sys.stderr.write("Output (myspell aff) encoding not specified; "
                             "assuming the same as input '%s'.\n" % dst_enc)

    conv = AffixTable(ispell_aff_file, src_enc)
    conv.dump_myspell_aff(myspell_cfg_file, dst_enc, sort_aff)


if __name__ == '__main__':
    main()
